enum SequenceStatus {
  IDLE,
  RUNNING,
  PAUSED
}

class Display {
  private parentElement : HTMLUListElement;
  private canvas : HTMLCanvasElement;
  private ctx : CanvasRenderingContext2D;
  private screen : any;
  constructor () {
    this.parentElement = document.getElementById('display') as HTMLUListElement;
    this.create();
  }
  private create () {
    this.canvas = this.parentElement.getElementsByTagName('canvas')[0];
    this.ctx = this.canvas.getContext('2d');
    this.screen = {
      width : parseInt((document.getElementById('width') as HTMLInputElement).value),
      height : parseInt((document.getElementById('height') as HTMLInputElement).value)
    }
    this.updateSize();
  }
  private updateSize () {
    const w : number = this.parentElement.clientWidth - 12;
    const h : number = this.parentElement.clientHeight - 12;
    const clientRatio : number = w / h;
    const screenRatio : number = this.screen.width / this.screen.height;
    let val : number;
    let offset : number;
    console.log(`${w},${h}`);
    this.canvas.width = w;
    this.canvas.height = h;
    this.ctx.strokeStyle = "rgb(0, 0, 255)";
    
    if (screenRatio > clientRatio) {
      val = Math.floor(w / screenRatio);
      offset = Math.round((h - val) / 2);
      this.ctx.rect(0, offset + 1, w, val);
    } else {
      val = Math.round(h * screenRatio);
      offset = Math.round((w - val) / 2);
      this.ctx.rect(offset, 1, val, h - 1);
    }

    this.ctx.stroke();
  }
  public update (msg : Message) {

  }
}

class Client {
  private display : Display;
  private client : WebSocket;
  private connected : boolean = false;
  private progress : HTMLProgressElement;

  constructor () {
    let uri : string = 'ws://localhost:8082';
    this.progress = document.getElementById('progress') as HTMLProgressElement;
    this.client = new WebSocket(uri);
    this.display = new Display();
    this.client.onopen = this.onOpen.bind(this);
    this.client.onclose = this.onClose.bind(this);
    this.client.onmessage = this.onMessage.bind(this);
    (document.getElementById('sequenceForm') as HTMLFormElement ).reset();
    (document.getElementById('sequenceCtrlForm') as HTMLFormElement ).reset();
    (document.getElementById('manualCtrlForm') as HTMLFormElement ).reset();
    this.disableClass('sequenceCtrl');
    this.disableClass('manualCtrl');
  }

  private onMessage (event : any) {
    const msg : Message = JSON.parse(event.data) as Message;
    if (typeof msg.cmd !== 'undefined' && msg.cmd !== null) {
      this.cmd(msg);
    }
  }

  private onOpen (event : any) {
    console.log('Connected');
    this.connected = true;
    this.active();
    this.enableClass('manualCtrl');
  }

  private onClose (event : any) {
    console.log('Disconnected');
    this.connected = false;
    this.inactive();
  }

  private setSequence(state : State) {
    this.setProgress(state.sequence);
    this.setStatus(state.sequence);
    this.setDisplay(state);
    (document.getElementById('sequence') as HTMLSelectElement ).value = state.sequence.hash;
  }

  private setStatus (sequence : SequenceState) {
    let status : string;
    switch (sequence.status) {
      case SequenceStatus.IDLE :
        status = 'Idle';
        break;
      case SequenceStatus.RUNNING :
        status = 'Running';
        break;
      case SequenceStatus.PAUSED :
        status = 'Paused';
        break;
      default :
        status = 'Unknown State';
    }
    document.getElementById('sequenceStatus').innerText = status;
    document.getElementById('sequenceName').innerText = sequence.name;
    document.getElementById('sequenceLength').innerText = `Sequence Length: ${sequence.frames}`;
  }

  private setProgress (sequence : SequenceState) {
    const percent : number = sequence.progress * 100.0;
    if (this.progress !== null) {
      this.progress.value = percent;
      this.progress.innerText = `${Math.floor(percent)}%`;
    }
    document.getElementById('sequenceProgress').innerText = `Progress: ${Math.round(sequence.progress)}%`;
  }

  private setDisplay (state : State) {
    console.dir(state);
  }

  private cmd (msg : Message) {
    switch (msg.cmd) {
      case 'open' :
        this.receiveCameraOpen();
        break;
      case 'close' :
        this.receiveCameraClose();
        break;
      case 'select' :
        this.receiveSelect(msg);
        break;
      default:
        console.warn(`No command "${msg.cmd}"`);
        break;
    }
  }

  public disableClass (className : string) {
    console.log(`Disabling class: ${className}`);
    document.querySelectorAll(`.${className}`).forEach((el : HTMLButtonElement) => {
      el.disabled = true;
    });
  }

  public enableClass (className : string) {
    console.log(`Enabling class: ${className}`);
    document.querySelectorAll(`.${className}`).forEach((el : HTMLButtonElement) => {
      el.disabled = false;
    });
  }

  public sendCameraOpen () {
    console.log('send camera open');
    this.disableClass('manualCtrl');
    this.client.send(JSON.stringify({ cmd : 'open' }));
  }

  private receiveCameraOpen () {
    console.log('got camera open');
    this.enableClass('manualCtrl');
  }

  public sendCameraClose () {
    console.log('send camera close');
    this.disableClass('manualCtrl');
    this.client.send(JSON.stringify({ cmd : 'close' }));
  }

  private receiveCameraClose () {
    console.log('got camera close');
    this.enableClass('manualCtrl');
  }

  public sendSelect () {
    const hash : string = (document.getElementById('sequence') as HTMLSelectElement ).value;
    let msg : Message;
    if (hash === '- Select Image Sequence -') {
      return;
    }
    msg = { cmd : 'select', state : { sequence : { hash } } };
    console.log('send select');
    console.log(hash)
    this.client.send(JSON.stringify(msg));
  }

  private receiveSelect (msg : Message) {
    console.log('got select');
    console.dir(msg);
    this.enableClass('sequenceCtrl');
    this.setSequence(msg.state);
  }

  private receiveUpdate (msg : Message) {

  }

  public fullscreen () {
    if (!document.fullscreenElement) {
      document.documentElement.requestFullscreen();
    } else {
      this.exitFullscreen();
    }
  }

  public exitFullscreen () {
    if (document.fullscreenElement) {
      document.exitFullscreen()
    }
  }

  private active () {
    document.getElementById('overlay').classList.add('active');
  }
  private inactive () {
    document.getElementById('overlay').classList.remove('active');
  }
}

const client : Client = new Client();