var SequenceStatus;
(function (SequenceStatus) {
    SequenceStatus[SequenceStatus["IDLE"] = 0] = "IDLE";
    SequenceStatus[SequenceStatus["RUNNING"] = 1] = "RUNNING";
    SequenceStatus[SequenceStatus["PAUSED"] = 2] = "PAUSED";
})(SequenceStatus || (SequenceStatus = {}));
class Display {
    constructor() {
        this.parentElement = document.getElementById('display');
        this.create();
    }
    create() {
        this.canvas = this.parentElement.getElementsByTagName('canvas')[0];
        this.ctx = this.canvas.getContext('2d');
        this.screen = {
            width: parseInt(document.getElementById('width').value),
            height: parseInt(document.getElementById('height').value)
        };
        this.updateSize();
    }
    updateSize() {
        const w = this.parentElement.clientWidth - 12;
        const h = this.parentElement.clientHeight - 12;
        const clientRatio = w / h;
        const screenRatio = this.screen.width / this.screen.height;
        let val;
        let offset;
        console.log(`${w},${h}`);
        this.canvas.width = w;
        this.canvas.height = h;
        this.ctx.strokeStyle = "rgb(0, 0, 255)";
        if (screenRatio > clientRatio) {
            val = Math.floor(w / screenRatio);
            offset = Math.round((h - val) / 2);
            this.ctx.rect(0, offset + 1, w, val);
        }
        else {
            val = Math.round(h * screenRatio);
            offset = Math.round((w - val) / 2);
            this.ctx.rect(offset, 1, val, h - 1);
        }
        this.ctx.stroke();
    }
    update(msg) {
    }
}
class Client {
    constructor() {
        this.connected = false;
        let uri = 'ws://localhost:8082';
        this.progress = document.getElementById('progress');
        this.client = new WebSocket(uri);
        this.display = new Display();
        this.client.onopen = this.onOpen.bind(this);
        this.client.onclose = this.onClose.bind(this);
        this.client.onmessage = this.onMessage.bind(this);
        document.getElementById('sequenceForm').reset();
        document.getElementById('sequenceCtrlForm').reset();
        document.getElementById('manualCtrlForm').reset();
        this.disableClass('sequenceCtrl');
        this.disableClass('manualCtrl');
    }
    onMessage(event) {
        const msg = JSON.parse(event.data);
        if (typeof msg.cmd !== 'undefined' && msg.cmd !== null) {
            this.cmd(msg);
        }
    }
    onOpen(event) {
        console.log('Connected');
        this.connected = true;
        this.active();
        this.enableClass('manualCtrl');
    }
    onClose(event) {
        console.log('Disconnected');
        this.connected = false;
        this.inactive();
    }
    setSequence(state) {
        this.setProgress(state.sequence);
        this.setStatus(state.sequence);
        this.setDisplay(state);
        document.getElementById('sequence').value = state.sequence.hash;
    }
    setStatus(sequence) {
        let status;
        switch (sequence.status) {
            case SequenceStatus.IDLE:
                status = 'Idle';
                break;
            case SequenceStatus.RUNNING:
                status = 'Running';
                break;
            case SequenceStatus.PAUSED:
                status = 'Paused';
                break;
            default:
                status = 'Unknown State';
        }
        document.getElementById('sequenceStatus').innerText = status;
        document.getElementById('sequenceName').innerText = sequence.name;
        document.getElementById('sequenceLength').innerText = `Sequence Length: ${sequence.frames}`;
    }
    setProgress(sequence) {
        const percent = sequence.progress * 100.0;
        if (this.progress !== null) {
            this.progress.value = percent;
            this.progress.innerText = `${Math.floor(percent)}%`;
        }
        document.getElementById('sequenceProgress').innerText = `Progress: ${Math.round(sequence.progress)}%`;
    }
    setDisplay(state) {
        console.dir(state);
    }
    cmd(msg) {
        switch (msg.cmd) {
            case 'open':
                this.receiveCameraOpen();
                break;
            case 'close':
                this.receiveCameraClose();
                break;
            case 'select':
                this.receiveSelect(msg);
                break;
            default:
                console.warn(`No command "${msg.cmd}"`);
                break;
        }
    }
    disableClass(className) {
        console.log(`Disabling class: ${className}`);
        document.querySelectorAll(`.${className}`).forEach((el) => {
            el.disabled = true;
        });
    }
    enableClass(className) {
        console.log(`Enabling class: ${className}`);
        document.querySelectorAll(`.${className}`).forEach((el) => {
            el.disabled = false;
        });
    }
    sendCameraOpen() {
        console.log('send camera open');
        this.disableClass('manualCtrl');
        this.client.send(JSON.stringify({ cmd: 'open' }));
    }
    receiveCameraOpen() {
        console.log('got camera open');
        this.enableClass('manualCtrl');
    }
    sendCameraClose() {
        console.log('send camera close');
        this.disableClass('manualCtrl');
        this.client.send(JSON.stringify({ cmd: 'close' }));
    }
    receiveCameraClose() {
        console.log('got camera close');
        this.enableClass('manualCtrl');
    }
    sendSelect() {
        const hash = document.getElementById('sequence').value;
        let msg;
        if (hash === '- Select Image Sequence -') {
            return;
        }
        msg = { cmd: 'select', state: { sequence: { hash } } };
        console.log('send select');
        console.log(hash);
        this.client.send(JSON.stringify(msg));
    }
    receiveSelect(msg) {
        console.log('got select');
        console.dir(msg);
        this.enableClass('sequenceCtrl');
        this.setSequence(msg.state);
    }
    receiveUpdate(msg) {
    }
    fullscreen() {
        if (!document.fullscreenElement) {
            document.documentElement.requestFullscreen();
        }
        else {
            this.exitFullscreen();
        }
    }
    exitFullscreen() {
        if (document.fullscreenElement) {
            document.exitFullscreen();
        }
    }
    active() {
        document.getElementById('overlay').classList.add('active');
    }
    inactive() {
        document.getElementById('overlay').classList.remove('active');
    }
}
const client = new Client();
//# sourceMappingURL=index.js.map